from flask import Blueprint, request
from zope import component

from .utils import authz_restrict_to_staff, authz_restrict_to_syscom, \
    user_is_in_group, requires_authentication_no_realm, \
    create_streaming_response, create_sync_response
from ceo_common.errors import UserNotFoundError
from ceo_common.interfaces import ILDAPService
from ceod.transactions.members import (
    AddMemberTransaction,
    ModifyMemberTransaction,
    RenewMemberTransaction,
    ResetPasswordTransaction,
)

bp = Blueprint('members', __name__)


@bp.route('/', methods=['POST'], strict_slashes=False)
@authz_restrict_to_staff
def create_user():
    body = request.get_json(force=True)
    txn = AddMemberTransaction(
        uid=body['uid'],
        cn=body['cn'],
        program=body.get('program'),
        terms=body.get('terms'),
        non_member_terms=body.get('non_member_terms'),
        forwarding_addresses=body.get('forwarding_addresses'),
    )
    return create_streaming_response(txn)


@bp.route('/<username>')
@requires_authentication_no_realm
def get_user(auth_user: str, username: str):
    get_forwarding_addresses = False
    if auth_user == username or user_is_in_group(auth_user, 'syscom'):
        get_forwarding_addresses = True
    ldap_srv = component.getUtility(ILDAPService)
    try:
        user = ldap_srv.get_user(username)
        return user.to_dict(get_forwarding_addresses)
    except UserNotFoundError:
        return {
            'error': 'user not found'
        }, 404


@bp.route('/<username>', methods=['PATCH'])
@requires_authentication_no_realm
def patch_user(auth_user: str, username: str):
    if not (auth_user == username or user_is_in_group(auth_user, 'syscom')):
        return {
            'error': "You are not authorized to modify other users' attributes"
        }, 403
    body = request.get_json(force=True)
    txn = ModifyMemberTransaction(
        username,
        login_shell=body.get('login_shell'),
        forwarding_addresses=body.get('forwarding_addresses'),
    )
    return create_streaming_response(txn)


@bp.route('/<username>/renew', methods=['POST'])
@authz_restrict_to_staff
def renew_user(username: str):
    body = request.get_json(force=True)
    txn = RenewMemberTransaction(
        username,
        terms=body.get('terms'),
        non_member_terms=body.get('non_member_terms'),
    )
    return create_sync_response(txn)


@bp.route('/<username>/pwreset', methods=['POST'])
@authz_restrict_to_syscom
def reset_user_password(username: str):
    txn = ResetPasswordTransaction(username)
    return create_sync_response(txn)
